--- Main script executed by nmap to make device verification or execute provision
-- This actions required to make automatic provision via HTTP requests.
--  
-- Examples of the used commands from WEB interface to execute this script:
-- $ nmap --script /var/www/provision/discovery.nse -sS -n --max-retries 2 --max-hostgroup 1 --max-rtt-timeout 1750ms --initial-rtt-timeout 1250ms --open -p 80,443 -oG - <DEVICE IP>
-- $ nmap --script /var/www/provision/discovery.nse -sS -n --max-retries 2 --max-hostgroup 1 --max-rtt-timeout 1750ms --initial-rtt-timeout 1250ms  --open -p 80,443 -oG - --script-args action=provision <DEVICE IP>
--
-- @file    discovery.nse 
-- @author  Vyacheslav Khudan
-- @created 2017/10/01
-- 
-- @note To make debug of the discovery.nse arguments required manual command executing and print funciton with output to stdout
-- @doc  https://nmap.org/book/nse-api.html

description = "Discovery hosts"
categories  = {}
portrule    = require"shortport".port_or_service(
    {"80"}, {"http"}, {"tcp"}, {"open","open|filtered"}
)
action      = function(host, port)
    local LUAVersion = _VERSION or ""
    
    -- try to check secure port
    local SSL_PORT = 443
    local socket   = nmap.new_socket()
    if socket:connect(host.ip, SSL_PORT, "tcp") then
        port.number  = SSL_PORT
        port.service = "https"
    end
    socket:close()
    
    -- processing scanner (PBX) IP address
    host.ip_src = host.ip_src and require"packet".toip(host.bin_ip_src)
    if not host.ip_src and host.ip and port.number then
        local socket = nmap.new_socket()
        socket:connect(host.ip, port.number)
        local _, ip_src = socket:get_info()
        socket:close()
        host.ip_src = ip_src
    end
    if not host.ip_src then
        return false
    end
    
    -- processing MAC address of the destination host
    host.mac_addr_str = host.mac_addr and require"stdnse".tohex(host.mac_addr) or ""
    
    -- running main procedure to execute standard interpreter
    local scriptDir = "/var/www/provision"
    local args      = {host = host, port = port, args = nmap.registry.args}
          args      = LUAVersion:find("5.1") and require"dkjson".encode(args) or require"json".generate(args) 
          args      = require"base64".enc(args)
    
    local cmd = string.format("lua5.1 %s/discovery.lua %s", scriptDir, args)
    return os.execute(cmd)
end
